@@ -16,6 +16,7 @@
 # Manage jobs in Jenkins server
 
 import os
+import sys
 import hashlib
 import yaml
 import json
@@ -34,6 +35,41 @@ logger = logging.getLogger(__name__)
 MAGIC_MANAGE_STRING = "<!-- Managed by Jenkins Job Builder -->"
 
 
+# Python <= 2.7.3's minidom toprettyxml produces broken output by adding
+# extraneous whitespace around data. This patches the broken implementation
+# with one taken from Python > 2.7.3.
+def writexml(self, writer, indent="", addindent="", newl=""):
+    # indent = current indentation
+    # addindent = indentation to add to higher levels
+    # newl = newline string
+    writer.write(indent + "<" + self.tagName)
+
+    attrs = self._get_attributes()
+    a_names = attrs.keys()
+    a_names.sort()
+
+    for a_name in a_names:
+        writer.write(" %s=\"" % a_name)
+        minidom._write_data(writer, attrs[a_name].value)
+        writer.write("\"")
+    if self.childNodes:
+        writer.write(">")
+        if (len(self.childNodes) == 1 and
+                self.childNodes[0].nodeType == minidom.Node.TEXT_NODE):
+            self.childNodes[0].writexml(writer, '', '', '')
+        else:
+            writer.write(newl)
+            for node in self.childNodes:
+                node.writexml(writer, indent + addindent, addindent, newl)
+            writer.write(indent)
+        writer.write("</%s>%s" % (self.tagName, newl))
+    else:
+        writer.write("/>%s" % (newl))
+
+if sys.version_info[:3] <= (2, 7, 3):
+    minidom.Element.writexml = writexml
+
+
 def deep_format(obj, paramdict):
     """Apply the paramdict via str.format() to all string objects found within
        the supplied obj. Lists and dicts are traversed recursively."""
@@ -84,6 +120,10 @@ class YamlParser(object):
     def parse(self, fn):
         data = yaml.load(open(fn))
         if data:
+            if not isinstance(data, list):
+                raise JenkinsJobsException(
+                    "The topmost collection in file '{fname}' must be a list,"
+                    " not a {cls}".format(fname=fn, cls=type(data)))
             for item in data:
                 cls, dfn = item.items()[0]
                 group = self.data.get(cls, {})
@@ -239,8 +279,8 @@ class YamlParser(object):
 
     def getXMLForJob(self, data):
         kind = data.get('project-type', 'freestyle')
-        data["description"] = data.get("description", "") + \
-            self.get_managed_string()
+        data["description"] = (data.get("description", "") +
+                               self.get_managed_string()).lstrip()
         for ep in pkg_resources.iter_entry_points(
                 group='jenkins_jobs.projects', name=kind):
             Mod = ep.load()
@@ -263,6 +303,8 @@ class YamlParser(object):
 
 
 class ModuleRegistry(object):
+    entry_points_cache = {}
+
     def __init__(self, config):
         self.modules = []
         self.modules_by_component_type = {}
@@ -331,16 +373,24 @@ class ModuleRegistry(object):
             component_data = {}
 
         # Look for a component function defined in an entry point
-        eps = list(pkg_resources.iter_entry_points(
-            group='jenkins_jobs.{0}'.format(component_list_type), name=name))
+        cache_key = '%s:%s' % (component_list_type, name)
+        eps = ModuleRegistry.entry_points_cache.get(cache_key)
+        if eps is None:
+            eps = list(pkg_resources.iter_entry_points(
+                       group='jenkins_jobs.{0}'.format(component_list_type),
+                       name=name))
+            if len(eps) > 1:
+                raise JenkinsJobsException(
+                    "Duplicate entry point found for component type: '{0}',"
+                    "name: '{1}'".format(component_type, name))
+            elif len(eps) == 1:
+                ModuleRegistry.entry_points_cache[cache_key] = eps
+                logger.debug("Cached entry point %s = %s", cache_key,
+                             ModuleRegistry.entry_points_cache[cache_key])
 
         if len(eps) == 1:
             func = eps[0].load()
             func(parser, xml_parent, component_data)
-        elif len(eps) > 1:
-            raise JenkinsJobsException("Duplicate entry point found for "
-                                       "component type: '{0}', name: '{1}'".
-                                       format(component_type, name))
         else:
             # Otherwise, see if it's defined as a macro
             component = parser.data.get(component_type, {}).get(name)
@@ -365,14 +415,9 @@ class XmlJob(object):
     def md5(self):
         return hashlib.md5(self.output()).hexdigest()
 
-    # Pretty printing ideas from
-    # http://stackoverflow.com/questions/749796/pretty-printing-xml-in-python
-    pretty_text_re = re.compile('>\n\s+([^<>\s].*?)\n\s+</', re.DOTALL)
-
     def output(self):
         out = minidom.parseString(XML.tostring(self.xml))
-        out = out.toprettyxml(indent='  ')
-        return self.pretty_text_re.sub('>\g<1></', out)
+        return out.toprettyxml(indent='  ', encoding='utf-8')
 
 
 class CacheStorage(object):
