@@ -22,14 +22,17 @@ in the :ref:`Job` definition.
 
 :Job Parameters:
     * **root-module**:
-        * **group-id** (`str`): GroupId. (required)
-        * **artifact-id** (`str`): ArtifactId. (required)
+        * **group-id** (`str`): GroupId.
+        * **artifact-id** (`str`): ArtifactId.
     * **root-pom** (`str`): The path to the pom.xml file. (defaults to pom.xml)
     * **goals** (`str`): Goals to execute. (required)
     * **maven-opts** (`str`): Java options to pass to maven (aka MAVEN_OPTS)
     * **maven-name** (`str`): Installation of maven which should be used.
       Not setting ``maven-name`` appears to use the first maven install
       defined in the global jenkins config.
+    * **private-repository** ('str'): Whether to use a private maven repository
+      Possible values are `default`, `local-to-workspace` and
+      `local-to-executor`.
     * **ignore-upstream-changes** (`bool`): Do not start a build whenever
       a SNAPSHOT dependency is built or not. (defaults to true)
     * **automatic-archiving** (`bool`): Activate automatic artifact archiving
@@ -59,15 +62,25 @@ import jenkins_jobs.modules.base
 class Maven(jenkins_jobs.modules.base.Base):
     sequence = 0
 
+    choices_private_repo = {
+        'default':
+        'hudson.maven.local_repo.DefaultLocalRepositoryLocator',
+        'local-to-workspace':
+        'hudson.maven.local_repo.PerJobLocalRepositoryLocator',
+        'local-to-executor':
+        'hudson.maven.local_repo.PerExecutorLocalRepositoryLocator',
+    }
+
     def root_xml(self, data):
-        if 'maven' not in data:
-            return None
         xml_parent = XML.Element('maven2-moduleset')
-        root_module = XML.SubElement(xml_parent, 'rootModule')
-        XML.SubElement(root_module, 'groupId').text = \
-            data['maven']['root-module']['group-id']
-        XML.SubElement(root_module, 'artifactId').text = \
-            data['maven']['root-module']['artifact-id']
+        if 'maven' not in data:
+            return xml_parent
+        if 'root-module' in data['maven']:
+            root_module = XML.SubElement(xml_parent, 'rootModule')
+            XML.SubElement(root_module, 'groupId').text = \
+                data['maven']['root-module']['group-id']
+            XML.SubElement(root_module, 'artifactId').text = \
+                data['maven']['root-module']['artifact-id']
         XML.SubElement(xml_parent, 'goals').text = data['maven']['goals']
 
         maven_opts = data['maven'].get('maven-opts')
@@ -78,6 +91,18 @@ class Maven(jenkins_jobs.modules.base.Base):
         if maven_name:
             XML.SubElement(xml_parent, 'mavenName').text = maven_name
 
+        private_repo = data['maven'].get('private-repository')
+        if private_repo:
+            if private_repo not in self.choices_private_repo.keys():
+                raise ValueError('Not a valid private-repository "%s", '
+                                 'must be one of "%s"' %
+                                 (private_repo,
+                                  ", ".join(self.choices_private_repo.keys())))
+            XML.SubElement(xml_parent,
+                           'localRepository',
+                           attrib={'class':
+                                   self.choices_private_repo[private_repo]})
+
         XML.SubElement(xml_parent, 'ignoreUpstremChanges').text = str(
             data['maven'].get('ignore-upstream-changes', True)).lower()
 
