@@ -26,22 +26,12 @@ SCMs plugin.
   :Macro: scm
   :Entry Point: jenkins_jobs.scm
 
-Example::
-
-  job:
-    name: test_job
-    scm:
-      -git:
-        url: https://example.com/project.git
-      -git:
-        url: https://example.org/otherproject.git
-        basedir: other
-
 """
 
 
 import xml.etree.ElementTree as XML
 import jenkins_jobs.modules.base
+from jenkins_jobs.errors import JenkinsJobsException
 
 
 def git(self, xml_parent, data):
@@ -51,6 +41,7 @@ def git(self, xml_parent, data):
     <https://wiki.jenkins-ci.org/display/JENKINS/Git+Plugin>`_
 
     :arg str url: URL of the git repository
+    :arg str credentials-id: ID of credentials to use to connect (optional)
     :arg str refspec: refspec to fetch
     :arg str name: name to fetch
     :arg list(str) branches: list of branch specifiers to build
@@ -58,6 +49,7 @@ def git(self, xml_parent, data):
       when polling for changes. (if polling is enabled)
     :arg list(str) included-regions: list of file/folders to include
     :arg list(str) excluded-regions: list of file/folders to exclude
+    :arg str local-branch: Checkout/merge to local branch
     :arg dict merge:
         :merge:
             * **remote** (`string`) - name of repo that contains branch to
@@ -80,6 +72,7 @@ def git(self, xml_parent, data):
     :arg bool wipe-workspace: Wipe out workspace before build
     :arg str browser: what repository browser to use (default '(Auto)')
     :arg str browser-url: url for the repository browser
+    :arg str browser-version: version of the repository browser (GitLab)
     :arg str project-name: project name in Gitblit and ViewGit repobrowser
     :arg str choosing-strategy: Jenkins class for selecting what to build
     :arg str git-config-name: Configure name for Git clone
@@ -101,16 +94,9 @@ def git(self, xml_parent, data):
         :inverse:
         :gerrit:
 
-    Example::
+    Example:
 
-      scm:
-        - git:
-          url: https://example.com/project.git
-          branches:
-            - master
-            - stable
-          browser: githubweb
-          browser-url: http://github.com/foo/example.git
+    .. literalinclude:: /../../tests/scm/fixtures/git001.yaml
     """
 
     # XXX somebody should write the docs for those with option name =
@@ -155,6 +141,8 @@ def git(self, xml_parent, data):
         refspec = '+refs/heads/*:refs/remotes/origin/*'
     XML.SubElement(huser, 'refspec').text = refspec
     XML.SubElement(huser, 'url').text = data['url']
+    if 'credentials-id' in data:
+        XML.SubElement(huser, 'credentialsId').text = data['credentials-id']
     xml_branches = XML.SubElement(scm, 'branches')
     branches = data.get('branches', ['**'])
     for branch in branches:
@@ -196,6 +184,10 @@ def git(self, xml_parent, data):
             xe.text = str(val).lower()
         else:
             xe.text = val
+
+    if 'local-branch' in data:
+        XML.SubElement(scm, 'localBranch').text = data['local-branch']
+
     browser = data.get('browser', 'auto')
     browserdict = {'githubweb': 'GithubWeb',
                    'fisheye': 'FisheyeGitRepositoryBrowser',
@@ -209,10 +201,10 @@ def git(self, xml_parent, data):
                    'viewgit': 'ViewGitWeb',
                    'auto': 'auto'}
     if browser not in browserdict:
-        raise Exception("Browser entered is not valid must be one of: " +
-                        "githubweb, fisheye, bitbucketweb, cgit, gitblit, " +
-                        "gitlab, gitoriousweb, gitweb, redmineweb, viewgit, " +
-                        "or auto")
+        raise JenkinsJobsException("Browser entered is not valid must be one "
+                                   "of: githubweb, fisheye, bitbucketweb, "
+                                   "cgit, gitblit, gitlab, gitoriousweb, "
+                                   "gitweb, redmineweb, viewgit, or auto")
     if browser != 'auto':
         bc = XML.SubElement(scm, 'browser', {'class':
                             'hudson.plugins.git.browser.' +
@@ -221,6 +213,9 @@ def git(self, xml_parent, data):
         if browser in ['gitblit', 'viewgit']:
             XML.SubElement(bc, 'projectName').text = str(
                 data.get('project-name', ''))
+        if browser == 'gitlab':
+            XML.SubElement(bc, 'version').text = str(
+                data.get('browser-version', '0.0'))
 
 
 def repo(self, xml_parent, data):
@@ -248,26 +243,9 @@ def repo(self, xml_parent, data):
     :arg str local-manifest: Contents of .repo/local_manifest.xml, written
              prior to calling sync (optional)
 
-    Example::
+    Example:
 
-      scm:
-        - repo:
-            manifest-url: https://example.com/project/
-            manifest-branch: stable
-            manifest-file: repo.xml
-            manifest-group: drivers
-            destination-dir: build
-            repo-url: https://internal.net/projects/repo
-            mirror-dir: ~/git/project/
-            jobs: 3
-            current-branch: false
-            quiet: false
-            local-manifest: |
-              <?xml version="1.0" encoding="UTF-8"?>
-              <manifest>
-                <project path="external/project" name="org/project"
-                  remote="gerrit" revision="master" />
-              </manifest>
+    .. literalinclude:: /../../tests/scm/fixtures/repo001.yaml
     """
 
     scm = XML.SubElement(xml_parent,
@@ -277,7 +255,7 @@ def repo(self, xml_parent, data):
         XML.SubElement(scm, 'manifestRepositoryUrl').text = \
             data['manifest-url']
     else:
-        raise Exception("Must specify a manifest url")
+        raise JenkinsJobsException("Must specify a manifest url")
 
     mapping = [
         # option, xml name, default value
@@ -354,7 +332,7 @@ def svn(self, xml_parent, data):
         XML.SubElement(module, 'remote').text = data['url']
         XML.SubElement(module, 'local').text = data.get('basedir', '.')
     else:
-        raise Exception("A top level url or repos list must exist")
+        raise JenkinsJobsException("A top level url or repos list must exist")
     updater = data.get('workspaceupdater', 'wipeworkspace')
     if updater == 'wipeworkspace':
         updaterclass = 'CheckoutUpdater'
