@@ -189,7 +189,7 @@ def gerrit(parser, xml_parent, data):
 
     You may select one or more gerrit events upon which to trigger.
     You must also supply at least one project and branch, optionally
-    more.  If you select the comment-added trigger, you should alse
+    more.  If you select the comment-added trigger, you should also
     indicate which approval category and value you want to trigger the
     job.
 
@@ -275,7 +275,7 @@ def gerrit(parser, xml_parent, data):
                                 ('gerrit-build-failed-verified-value',
                                  'gerritBuildFailedVerifiedValue'),
                                 ('gerrit-build-successful-codereview-value',
-                                 'gerritBuildSuccesfulCodereviewValue'),
+                                 'gerritBuildSuccessfulCodereviewValue'),
                                 ('gerrit-build-failed-codereview-value',
                                  'gerritBuildFaiedCodeReviewValue')]:
             if data.get(yamlkey) is not None:
@@ -348,24 +348,22 @@ def github_pull_request(parser, xml_parent, data):
     GitHub+pull+request+builder+plugin>`_
 
     :arg list admin-list: the users with admin rights (optional)
-    :arg string cron: cron syntax of when to run (optional)
     :arg list white-list: users whose pull requests build (optional)
     :arg list org-list: orgs whose users should be white listed (optional)
+    :arg string cron: cron syntax of when to run (optional)
+    :arg string trigger-phrase: when filled, commenting this phrase
+        in the pull request will trigger a build (optional)
+    :arg bool only-trigger-phrase: only commenting the trigger phrase
+        in the pull request will trigger a build (default false)
+    :arg bool github-hooks: use github hook (default false)
+    :arg bool permit-all: build every pull request automatically
+        without asking (default false)
+    :arg bool auto-close-on-fail: close failed pull request automatically
+        (default false)
 
-    Example::
+    Example:
 
-      triggers:
-        - github-pull-request:
-            admin-list:
-              - user1
-              - user2
-            cron: * * * * *
-            white-list:
-              - user3
-              - user4
-            org-list:
-              - org1
-              - org2
+    .. literalinclude:: /../../tests/triggers/fixtures/github-pull-request.yaml
     """
     ghprb = XML.SubElement(xml_parent, 'org.jenkinsci.plugins.ghprb.'
                            'GhprbTrigger')
@@ -377,6 +375,16 @@ def github_pull_request(parser, xml_parent, data):
     org_string = "\n".join(data.get('org-list', []))
     XML.SubElement(ghprb, 'orgslist').text = org_string
     XML.SubElement(ghprb, 'cron').text = data.get('cron', '')
+    XML.SubElement(ghprb, 'triggerPhrase').text = \
+        data.get('trigger-phrase', '')
+    XML.SubElement(ghprb, 'onlyTriggerPhrase').text = str(
+        data.get('only-trigger-phrase', False)).lower()
+    XML.SubElement(ghprb, 'useGitHubHooks').text = str(
+        data.get('github-hooks', False)).lower()
+    XML.SubElement(ghprb, 'permitAll').text = str(
+        data.get('permit-all', False)).lower()
+    XML.SubElement(ghprb, 'autoCloseFailedPullRequests').text = str(
+        data.get('auto-close-on-fail', False)).lower()
 
 
 def build_result(parser, xml_parent, data):
@@ -446,6 +454,43 @@ def build_result(parser, xml_parent, data):
             XML.SubElement(model_checked, 'checked').text = result_dict[result]
 
 
+def script(parser, xml_parent, data):
+    """yaml: script
+    Triggers the job using shell or batch script.
+    Requires the Jenkins `ScriptTrigger Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/ScriptTrigger+Plugin>`_
+
+    :arg str label: Restrict where the polling should run. (default '')
+    :arg str script: A shell or batch script. (default '')
+    :arg str cron: cron syntax of when to run (default '')
+    :arg bool enable-concurrent:  Enables triggering concurrent builds.
+                                  (default false)
+    :arg int exit-code:  If the exit code of the script execution returns this
+                         expected exit code, a build is scheduled. (default 0)
+
+    Example:
+
+    .. literalinclude:: /../../tests/triggers/fixtures/script.yaml
+    """
+    data = data if data else {}
+    st = XML.SubElement(
+        xml_parent,
+        'org.jenkinsci.plugins.scripttrigger.ScriptTrigger'
+    )
+    label = data.get('label')
+
+    XML.SubElement(st, 'script').text = str(data.get('script', ''))
+    XML.SubElement(st, 'scriptFilePath').text = str(
+        data.get('script-file-path', ''))
+    XML.SubElement(st, 'spec').text = str(data.get('cron', ''))
+    XML.SubElement(st, 'labelRestriction').text = str(bool(label)).lower()
+    if label:
+        XML.SubElement(st, 'triggerLabel').text = label
+    XML.SubElement(st, 'enableConcurrentBuild').text = str(
+        data.get('enable-concurrent', False)).lower()
+    XML.SubElement(st, 'exitCode').text = str(data.get('exit-code', 0))
+
+
 class Triggers(jenkins_jobs.modules.base.Base):
     sequence = 50
 
